@@ -1,4 +1,5 @@
 import numpy as np
+from astropy.nddata import StdDevUncertainty, VarianceUncertainty, InverseVariance
 
 from ..manipulation import (FluxConservingResampler,
                             LinearInterpolatedResampler,
@@ -7,7 +8,34 @@ from ..spectra.spectrum1d import Spectrum1D
 
 __all__ = ['template_match', 'template_redshift']
 
-def _normalize_for_template_matching(observed_spectrum, template_spectrum):
+
+def _uncertainty_to_standard_deviation(uncertainty):
+    """
+    Convenience function to convert other uncertainty types to standard deviation,
+    for consistency in calculations elsewhere.
+
+    Parameters
+    ----------
+    uncertainty : :class:`~astropy.nddata.NDUncertainty`
+        The input uncertainty
+
+    Returns
+    -------
+    :class:`~numpy.ndarray`
+        The array of standard deviation values.
+
+    """
+    if uncertainty is not None:
+        if isinstance(uncertainty, StdDevUncertainty):
+            stddev = uncertainty.array
+        elif isinstance(uncertainty, VarianceUncertainty):
+            stddev = np.sqrt(uncertainty.array)
+        elif isinstance(uncertainty, InverseVariance):
+            stddev = 1 / np.sqrt(uncertainty.array)
+
+        return stddev
+
+def _normalize_for_template_matching(observed_spectrum, template_spectrum, stddev=None):
     """
     Calculate a scale factor to be applied to the template spectrum so the
     total flux in both spectra will be the same.
@@ -26,10 +54,10 @@ def _normalize_for_template_matching(observed_spectrum, template_spectrum):
         A float which will normalize the template spectrum's flux so that it
         can be compared to the observed spectrum.
     """
-    num = np.sum((observed_spectrum.flux*template_spectrum.flux)/
-                 (observed_spectrum.uncertainty.array**2))
-    denom = np.sum((template_spectrum.flux/
-                    observed_spectrum.uncertainty.array)**2)
+    if stddev is None:
+        stddev = _uncertainty_to_standard_deviation(observed_spectrum.uncertainty)
+    num = np.sum((observed_spectrum.flux*template_spectrum.flux) / (stddev**2))
+    denom = np.sum((template_spectrum.flux / stddev)**2)
 
     return num/denom
 
@@ -88,16 +116,20 @@ def _chi_square_for_templates(observed_spectrum, template_spectrum, resample_met
         template_obswavelength = fluxc_resample(template_spectrum,
                                                 observed_spectrum.spectral_axis)
 
+    # Convert the uncertainty to standard deviation if needed
+    stddev = _uncertainty_to_standard_deviation(observed_spectrum.uncertainty)
+
     # Normalize spectra
     normalization = _normalize_for_template_matching(observed_spectrum,
-                                                     template_obswavelength)
+                                                     template_obswavelength,
+                                                     stddev)
 
     # Numerator
     num_right = normalization * template_obswavelength.flux
     num = observed_spectrum.flux - num_right
 
     # Denominator
-    denom = observed_spectrum.uncertainty.array * observed_spectrum.flux.unit
+    denom = stddev * observed_spectrum.flux.unit
 
     # Get chi square
     result = (num/denom)**2
@@ -132,9 +164,6 @@ def template_match(observed_spectrum, spectral_templates,
     resample_method : `string`
         Three resample options: flux_conserving, linear_interpolated, and spline_interpolated.
         Anything else does not resample the spectrum.
-    known_redshift: `float`
-        If the user knows the redshift they want to apply to the spectrum/spectra within spectral_templates,
-        then this redshift can be applied to each template before attempting the match.
     redshift : 'float', `int`, `list`, `tuple`, 'numpy.array`
         If the user knows the redshift they want to apply to the spectrum/spectra within spectral_templates,
         then this float or int value redshift can be applied to each template before attempting the match.
@@ -146,27 +175,33 @@ def template_match(observed_spectrum, spectral_templates,
     -------
     normalized_template_spectrum : :class:`~specutils.Spectrum1D`
         The template spectrum that has been normalized.
-    chi2 : `float`
-        The chi2 of the flux of the observed_spectrum and the flux of the
-        normalized template spectrum.
+    redshift : `None` or `float`
+        The value of the redshift that provides the smallest chi2.
     smallest_chi_index : `int`
         The index of the spectrum with the smallest chi2 in spectral templates.
+    chi2_min : `float`
+        The chi2 of the flux of the observed_spectrum and the flux of the
+        normalized template spectrum.
     chi2_list : `list`
         A list with all chi2 values found for each template spectrum.
     """
+    final_redshift = None
+
     if hasattr(spectral_templates, 'flux') and len(spectral_templates.flux.shape) == 1:
 
         # Account for redshift if provided
         chi2_list = []
         if redshift is not None:
-            _, redshifted_spectrum, chi2_list = template_redshift(observed_spectrum, spectral_templates,
-                                                               redshift=redshift)
-            spectral_templates = redshifted_spectrum
+            results = template_redshift(observed_spectrum, spectral_templates, redshift=redshift)
+            redshifted_spectrum, final_redshift, normalized_spectral_template, chi2, chi2_inner_list = results # noqa
 
-        normalized_spectral_template, chi2 = _chi_square_for_templates(
-            observed_spectrum, spectral_templates, resample_method)
+        else:
+            normalized_spectral_template, chi2 = _chi_square_for_templates(
+                observed_spectrum, spectral_templates, resample_method)
 
-        return normalized_spectral_template, chi2, 0, chi2_list
+        chi2_list.append(chi2)
+
+        return normalized_spectral_template, final_redshift, 0, chi2, chi2_list
 
     # At this point, the template spectrum is either a ``SpectrumCollection``
     # or a multi-dimensional``Spectrum1D``. Loop through the object and return
@@ -176,25 +211,30 @@ def template_match(observed_spectrum, spectral_templates,
     smallest_chi_spec = None
     chi2_list = []
 
+    final_redshift = None
+    temp_redshift = None
+
     for index, spectrum in enumerate(spectral_templates):
 
         # Account for redshift if provided
         if redshift is not None:
-            _, redshifted_spectrum, chi2_inner_list = template_redshift(
-                observed_spectrum, spectrum, redshift=redshift)
-            spectrum = redshifted_spectrum
+            results = template_redshift(observed_spectrum, spectrum, redshift=redshift)
+            redshifted_spectrum, temp_redshift, normalized_spectral_template, chi2, chi2_inner_list = results # noqa
 
             chi2_list.append(chi2_inner_list)
 
-        normalized_spectral_template, chi2 = _chi_square_for_templates(
-            observed_spectrum, spectrum, resample_method)
+        else:
+            normalized_spectral_template, chi2 = _chi_square_for_templates(
+                observed_spectrum, spectrum, resample_method)
 
         if chi2_min is None or chi2 < chi2_min:
             chi2_min = chi2
             smallest_chi_spec = normalized_spectral_template
             smallest_chi_index = index
 
-    return smallest_chi_spec, chi2_min, smallest_chi_index, chi2_list
+            final_redshift = temp_redshift
+
+    return smallest_chi_spec, final_redshift, smallest_chi_index, chi2_min, chi2_list
 
 
 def template_redshift(observed_spectrum, template_spectrum, redshift):
@@ -212,11 +252,15 @@ def template_redshift(observed_spectrum, template_spectrum, redshift):
 
     Returns
     -------
-    final_redshift : `float`
-        The best-fit redshift for template_spectrum to match the observed_spectrum.
     redshifted_spectrum: :class:`~specutils.Spectrum1D`
         A new Spectrum1D object which incorporates the template_spectrum with a spectral_axis
         that has been redshifted using the final_redshift.
+    final_redshift : `float`
+        The best-fit redshift for template_spectrum to match the observed_spectrum.
+    normalized_template_spectrum : :class:`~specutils.Spectrum1D`
+        The normalized spectrum template.
+    chi2_min: `float`
+        The smallest chi2 value that was found.
     chi2_list : `list`
         A list with the chi2 values corresponding to each input redshift value.
     """
@@ -231,10 +275,12 @@ def template_redshift(observed_spectrum, template_spectrum, redshift):
 
         # Create new redshifted spectrum and run it through the chi2 method
         redshifted_spectrum = Spectrum1D(spectral_axis=template_spectrum.spectral_axis*(1+rs),
-                        flux=template_spectrum.flux, uncertainty=template_spectrum.uncertainty,
+                                         flux=template_spectrum.flux,
+                                         uncertainty=template_spectrum.uncertainty,
                                          meta=template_spectrum.meta)
+
         normalized_spectral_template, chi2 = _chi_square_for_templates(
-                        observed_spectrum, redshifted_spectrum, "flux_conserving")
+            observed_spectrum, redshifted_spectrum, "flux_conserving")
 
         chi2_list.append(chi2)
 
@@ -243,4 +289,4 @@ def template_redshift(observed_spectrum, template_spectrum, redshift):
             chi2_min = chi2
             final_redshift = rs
 
-    return final_redshift, redshifted_spectrum, chi2_list
+    return redshifted_spectrum, final_redshift, normalized_spectral_template, chi2_min, chi2_list
