@@ -11,7 +11,7 @@ from astropy.nddata import StdDevUncertainty
 from asdf.tests.helpers import assert_roundtrip_tree
 import asdf
 
-from specutils import Spectrum1D, SpectrumList
+from specutils import Spectrum1D, SpectrumList, SpectralAxis
 
 
 def create_spectrum1d(xmin, xmax, uncertainty=None):
@@ -37,6 +37,13 @@ def test_asdf_spectrum1d_uncertainty(tmpdir):
     tree = dict(spectrum=spectrum)
     assert_roundtrip_tree(tree, tmpdir)
 
+@pytest.mark.xfail
+def test_asdf_spectralaxis(tmpdir):
+
+    wavelengths  = np.arange(5100, 5300) * 0.1 * u.nm
+    spectral_axis = SpectralAxis(wavelengths, bin_specification="edges")
+    tree = dict(spectral_axis=spectral_axis)
+    assert_roundtrip_tree(tree, tmpdir)
 
 def test_asdf_spectrumlist(tmpdir):
 
