@@ -582,13 +582,14 @@ def _jwst_s3d_loader(filename, **kwargs):
             flux = Quantity(flux_array, unit=flux_unit)
 
             # Get the wavelength array from the GWCS object which returns a
-            # tuple of (RA, Dec, lambda)
-            grid = grid_from_bounding_box(wcs.bounding_box)
-            _, _, lam = wcs(*grid)
-            _, _, lam_unit = wcs.output_frame.unit
+            # tuple of (RA, Dec, lambda).
+            # Since the spatial and spectral axes are orthogonal in s3d data,
+            # it is much faster to compute a slice down the spectral axis.
+            grid = grid_from_bounding_box(wcs.bounding_box)[:, :, 0, 0]
+            _, _, wavelength_array = wcs(*grid)
+            _, _, wavelength_unit = wcs.output_frame.unit
 
-            wavelength_array = lam[:, 0, 0]
-            wavelength = Quantity(wavelength_array, unit=lam_unit)
+            wavelength = Quantity(wavelength_array, unit=wavelength_unit)
 
             # Merge primary and slit headers and dump into meta
             slit_header = hdu.header
