@@ -1,7 +1,7 @@
-
 import astropy.units as u
 from astropy.io import fits
 from astropy.nddata import InverseVariance
+from astropy.wcs import WCS
 
 from ...spectra import Spectrum1D
 from ..registers import data_loader
@@ -33,8 +33,10 @@ def identify_manga_rss(origin, *args, **kwargs):
                 and hdulist[1].header["NAXIS"] == 2)
 
 
-@data_loader("MaNGA cube", identifier=identify_manga_cube, dtype=Spectrum1D,
-             extensions=['fits'])
+@data_loader(
+    "MaNGA cube", identifier=identify_manga_cube, dtype=Spectrum1D,
+    extensions=['fits'], priority=10,
+)
 def manga_cube_loader(file_obj, **kwargs):
     """
     Loader for MaNGA 3D rectified spectral data in FITS format.
@@ -53,13 +55,15 @@ def manga_cube_loader(file_obj, **kwargs):
 
     spaxel = u.Unit('spaxel', represents=u.pixel, doc='0.5" spatial pixel', parse_strict='silent')
     with read_fileobj_or_hdulist(file_obj, **kwargs) as hdulist:
-        spectrum = _load_manga_spectra(hdulist, per_unit=spaxel, transpose=True)
+        spectrum = _load_manga_spectra(hdulist, per_unit=spaxel)
 
     return spectrum
 
 
-@data_loader("MaNGA rss", identifier=identify_manga_rss, dtype=Spectrum1D,
-             extensions=['fits'])
+@data_loader(
+    "MaNGA rss", identifier=identify_manga_rss, dtype=Spectrum1D,
+    extensions=['fits'], priority=10,
+)
 def manga_rss_loader(file_obj, **kwargs):
     """
     Loader for MaNGA 2D row-stacked spectral data in FITS format.
@@ -83,13 +87,13 @@ def manga_rss_loader(file_obj, **kwargs):
     return spectrum
 
 
-def _load_manga_spectra(hdulist, per_unit=None, transpose=None):
+def _load_manga_spectra(hdulist, per_unit=None):
     """ Return a MaNGA Spectrum1D object
 
-    Returns a Spectrum1D object for a MaNGA data files.  Set
-    `transpose` kwarg to True for MaNGA cubes, as they are flipped relative
-    to what Spectrum1D expects.  Use the `per_unit` kwarg to indicate the
-    "spaxel" or "fiber" unit for cubes and rss files, respectively.
+    Returns a Spectrum1D object for a MaNGA data files. Use the `per_unit`
+    kwarg to indicate the "spaxel" or "fiber" unit for cubes and rss files,
+    respectively. Note that the spectral axis will automatically be moved to
+    be last during Spectrum1D initialization.
 
     Parameters
     ----------
@@ -97,8 +101,6 @@ def _load_manga_spectra(hdulist, per_unit=None, transpose=None):
         A MaNGA read astropy fits HDUList
     per_unit : astropy.units.Unit
         An astropy unit to divide the default flux unit by
-    transpose : bool
-        If True, transpose the data arrays
 
     Returns
     -------
@@ -110,18 +112,12 @@ def _load_manga_spectra(hdulist, per_unit=None, transpose=None):
         unit = unit / per_unit
 
     hdr = hdulist['PRIMARY'].header
-    wave = hdulist['WAVE'].data * u.angstrom
+    wcs = WCS(hdulist['FLUX'].header)
 
-    if transpose:
-        flux = hdulist['FLUX'].data.T * unit
-        ivar = InverseVariance(hdulist["IVAR"].data.T)
-        # SDSS masks are arrays of bit values storing multiple boolean conditions.
-        # Setting non-zero bit values to True to map to specutils standard
-        mask = hdulist['MASK'].data.T != 0
-    else:
-        flux = hdulist['FLUX'].data * unit
-        ivar = InverseVariance(hdulist["IVAR"].data)
-        mask = hdulist['MASK'].data != 0
+    flux = hdulist['FLUX'].data * unit
+    ivar = InverseVariance(hdulist["IVAR"].data)
+    # SDSS masks are arrays of bit values storing multiple boolean conditions.
+    mask = hdulist['MASK'].data != 0
 
-    return Spectrum1D(flux=flux, meta={'header': hdr}, spectral_axis=wave,
+    return Spectrum1D(flux=flux, meta={'header': hdr}, wcs=wcs,
                       uncertainty=ivar, mask=mask)
