@@ -1,4 +1,3 @@
-import logging
 import os
 
 from astropy.io import fits
@@ -26,8 +25,10 @@ def identify_muscles_sed(origin, *args, **kwargs):
                 hdulist[0].header.get('PROPOSID') == 13650)
 
 
-@data_loader(label="MUSCLES SED", identifier=identify_muscles_sed,
-             dtype=Spectrum1D, extensions=['fits'])
+@data_loader(
+    label="MUSCLES SED", identifier=identify_muscles_sed, dtype=Spectrum1D,
+    extensions=['fits'], priority=10,
+)
 def muscles_sed(file_obj, **kwargs):
     """
     Load spectrum from a MUSCLES Treasury Survey panchromatic SED FITS file.
