@@ -1,4 +1,3 @@
-import logging
 import os
 import _io
 
@@ -135,10 +134,12 @@ def tabular_fits_writer(spectrum, file_name, hdu=1, update_header=False, **kwarg
     wunit = u.Unit(kwargs.pop('wunit', spectrum.spectral_axis.unit))
     disp = spectrum.spectral_axis.to(wunit, equivalencies=u.spectral())
 
-    # Mapping of spectral_axis types to header TTYPE1
-    dispname = wunit.physical_type
+    # Mapping of spectral_axis types to header TTYPE1 (no "torque/work" types!)
+    dispname = str(wunit.physical_type)
     if dispname == "length":
         dispname = "wavelength"
+    elif "energy" in dispname:
+        dispname = "energy"
 
     # Add flux array and unit
     ftype = kwargs.pop('ftype', spectrum.flux.dtype)
@@ -154,18 +155,10 @@ def tabular_fits_writer(spectrum, file_name, hdu=1, update_header=False, **kwarg
         columns.append(unc.astype(ftype))
         colnames.append("uncertainty")
 
-    # For 2D data transpose from row-major format
-    ndim = 1
+    # For > 1D data transpose from row-major format
     for c in range(1, len(columns)):
         if columns[c].ndim > 1:
-            ndim = columns[c].ndim
             columns[c] = columns[c].T
-    if ndim > 1:
-        spec_shape = np.ones(ndim, dtype=np.int)
-        spec_shape[0] = -1
-        for c in range(len(columns)):
-            if columns[c].ndim == 1:
-                columns[c] = columns[c].reshape(spec_shape)
 
     tab = Table(columns, names=colnames, meta=header)
 
