@@ -4,6 +4,7 @@ import re
 import urllib
 import io
 import contextlib
+import logging
 
 from astropy.io import fits
 from astropy.table import Table
@@ -11,10 +12,11 @@ from astropy.nddata import StdDevUncertainty
 from astropy.utils.exceptions import AstropyUserWarning
 import astropy.units as u
 import warnings
-import logging
 
 from specutils.spectra import Spectrum1D, SpectrumCollection
 
+log = logging.getLogger(__name__)
+
 
 @contextlib.contextmanager
 def read_fileobj_or_hdulist(*args, **kwargs):
@@ -97,18 +99,17 @@ def spectrum_from_column_mapping(table, column_mapping, wcs=None):
             kwarg_val = u.Quantity(table[col_name], tab_unit)
 
             # Attempt to convert the table unit to the user-defined unit.
-            logging.debug("Attempting auto-convert of table unit '%s' to "
-                          "user-provided unit '%s'.", tab_unit, cm_unit)
+            log.debug("Attempting auto-convert of table unit '%s' to "
+                      "user-provided unit '%s'.", tab_unit, cm_unit)
 
             if not isinstance(cm_unit, u.Unit):
                 cm_unit = u.Unit(cm_unit)
-            if cm_unit.physical_type in ('length', 'frequency'):
+            if cm_unit.physical_type in ('length', 'frequency', 'energy'):
                 # Spectral axis column information
                 kwarg_val = kwarg_val.to(cm_unit, equivalencies=u.spectral())
-            elif 'spectral flux' in cm_unit.physical_type:
+            elif 'spectral flux' in str(cm_unit.physical_type):
                 # Flux/error column information
-                kwarg_val = kwarg_val.to(
-                    cm_unit, equivalencies=u.spectral_density(1 * u.AA))
+                kwarg_val = kwarg_val.to(cm_unit, equivalencies=u.spectral_density(1 * u.AA))
         elif tab_unit:
             # The user has provided no unit in the column mapping, so we
             # use the unit as defined in the table object.
@@ -187,7 +188,7 @@ def generic_spectrum_from_table(table, wcs=None, **kwargs):
                 table[c].to("AA", equivalencies=u.spectral())
                 found_column = c
                 break
-            except:
+            except Exception:
                 continue
 
         # If no success there, check for other possible length units
@@ -224,7 +225,7 @@ def generic_spectrum_from_table(table, wcs=None, **kwargs):
                 table[c].to("Jy", equivalencies=u.spectral_density(spec_ax))
                 found_column = c
                 break
-            except:
+            except Exception:
                 continue
 
         # If no success there, check for other possible flux units
@@ -330,5 +331,10 @@ def _fits_identify_by_name(origin, fileinp, *args,
         if fileinp.seekable():
             fileobj = fileinp
 
-    return (_spec_pattern.match(os.path.basename(filepath)) is not None and
-            fits.connect.is_fits(origin, filepath, fileobj, *args))
+    check = (_spec_pattern.match(os.path.basename(filepath)) is not None and
+             fits.connect.is_fits(origin, filepath, fileobj, *args))
+
+    if fileobj is not None:
+        fileobj.close()
+
+    return check
