@@ -1,22 +1,34 @@
 """
 A module containing the mechanics of the specutils io registry.
 """
+import inspect
 import os
-import logging
 import pathlib
 import sys
 from functools import wraps
+import logging
 
 from astropy.io import registry as io_registry
 
 from ..spectra import Spectrum1D, SpectrumList, SpectrumCollection
 
-
 __all__ = ['data_loader', 'custom_writer', 'get_loaders_by_extension', 'identify_spectrum_format']
 
+log = logging.getLogger(__name__)
+
+
+def _astropy_has_priorities():
+    """
+    Check if astropy has support for loader priorities
+    """
+    sig = inspect.signature(io_registry.register_reader)
+    if sig.parameters.get("priority") is not None:
+        return True
+    return False
+
 
 def data_loader(label, identifier=None, dtype=Spectrum1D, extensions=None,
-                priority=0):
+                priority=0, force=False):
     """
     Wraps a function that can be added to an `~astropy.io.registry` for custom
     file reading.
@@ -38,6 +50,10 @@ def data_loader(label, identifier=None, dtype=Spectrum1D, extensions=None,
     priority : int
         Set the priority of the loader. Currently influences the sorting of the
         returned loaders for a dtype.
+    force : bool, optional
+        Whether to override any existing function if already present.
+        Default is ``False``. Passed down to astropy registry.
+
     """
     def identifier_wrapper(ident):
         def wrapper(*args, **kwargs):
@@ -45,19 +61,26 @@ def data_loader(label, identifier=None, dtype=Spectrum1D, extensions=None,
             try:
                 return ident(*args, **kwargs)
             except Exception as e:
-                logging.debug("Tried to read this as {} file, but could not.".format(label))
-                logging.debug(e, exc_info=True)
+                log.debug("Tried to read this as {} file, but could not.".format(label))
+                log.debug(e, exc_info=True)
                 return False
         return wrapper
 
     def decorator(func):
-        io_registry.register_reader(label, dtype, func)
+        if _astropy_has_priorities():
+            io_registry.register_reader(
+                label, dtype, func, priority=priority, force=force,
+            )
+        else:
+            io_registry.register_reader(
+                label, dtype, func, force=force,
+            )
 
         if identifier is None:
             # If the identifier is not defined, but the extensions are, create
             # a simple identifier based off file extension.
             if extensions is not None:
-                logging.info("'{}' data loader provided for {} without "
+                log.info("'{}' data loader provided for {} without "
                              "explicit identifier. Creating identifier using "
                              "list of compatible extensions".format(
                                  label, dtype.__name__))
@@ -65,30 +88,21 @@ def data_loader(label, identifier=None, dtype=Spectrum1D, extensions=None,
                                                           for x in extensions])
             # Otherwise, create a dummy identifier
             else:
-                logging.warning("'{}' data loader provided for {} without "
+                log.warning("'{}' data loader provided for {} without "
                              "explicit identifier or list of compatible "
                              "extensions".format(label, dtype.__name__))
                 id_func = lambda *args, **kwargs: True
         else:
             id_func = identifier_wrapper(identifier)
 
-        io_registry.register_identifier(label, dtype, id_func)
+        io_registry.register_identifier(
+            label, dtype, id_func, force=force,
+        )
 
         # Include the file extensions as attributes on the function object
         func.extensions = extensions
 
-        # Include priority on the loader function attribute
-        func.priority = priority
-
-        # Sort the io_registry based on priority
-        sorted_loaders = sorted(io_registry._readers.items(),
-                                key=lambda item: getattr(item[1], 'priority', 0))
-
-        # Update the registry with the sorted dictionary
-        io_registry._readers.clear()
-        io_registry._readers.update(sorted_loaders)
-
-        logging.debug("Successfully loaded reader \"{}\".".format(label))
+        log.debug("Successfully loaded reader \"{}\".".format(label))
 
         # Automatically register a SpectrumList reader for any data_loader that
         # reads Spectrum1D objects. TODO: it's possible that this
@@ -101,9 +115,19 @@ def data_loader(label, identifier=None, dtype=Spectrum1D, extensions=None,
             load_spectrum_list.extensions = extensions
             load_spectrum_list.priority = priority
 
-            io_registry.register_reader(label, SpectrumList, load_spectrum_list)
-            io_registry.register_identifier(label, SpectrumList, id_func)
-            logging.debug("Created SpectrumList reader for \"{}\".".format(label))
+            if _astropy_has_priorities():
+                io_registry.register_reader(
+                    label, SpectrumList, load_spectrum_list, priority=priority,
+                    force=force,
+                )
+            else:
+                io_registry.register_reader(
+                    label, SpectrumList, load_spectrum_list, force=force,
+                )
+            io_registry.register_identifier(
+                label, SpectrumList, id_func, force=force,
+            )
+            log.debug("Created SpectrumList reader for \"{}\".".format(label))
 
         @wraps(func)
         def wrapper(*args, **kwargs):
@@ -112,9 +136,14 @@ def data_loader(label, identifier=None, dtype=Spectrum1D, extensions=None,
     return decorator
 
 
-def custom_writer(label, dtype=Spectrum1D):
+def custom_writer(label, dtype=Spectrum1D, priority=0, force=False):
     def decorator(func):
-        io_registry.register_writer(label, Spectrum1D, func)
+        if _astropy_has_priorities():
+            io_registry.register_writer(
+                label, dtype, func, priority=priority, force=force,
+            )
+        else:
+            io_registry.register_writer(label, dtype, func, force=force)
 
         @wraps(func)
         def wrapper(*args, **kwargs):
@@ -137,7 +166,16 @@ def get_loaders_by_extension(extension):
     loaders : list
         A list of loader names that are associated with the extension.
     """
-    return [fmt for (fmt, cls), func in io_registry._readers.items()
+    def _registered_readers():
+        # With the implementation of priorities support in the astropy registry
+        # loaders, astropy version 4.2 and up return a tuple of ``func`` and
+        # ``priority``, while versions < 4.2 return just the ``func`` object.
+        # This function ignores priorities when calling extension loaders.
+        return [((fmt, cls), func[0])
+                if isinstance(func, tuple) else ((fmt, cls), func)
+                for (fmt, cls), func in io_registry._readers.items()]
+
+    return [fmt for (fmt, cls), func in _registered_readers()
             if issubclass(cls, Spectrum1D) and
             func.extensions is not None and
             extension in func.extensions]
