@@ -9,8 +9,11 @@ from astropy.tests.helper import quantity_allclose
 from astropy.utils.exceptions import AstropyUserWarning
 
 from ..spectra import Spectrum1D, SpectralRegion
+from ..spectra.spectrum_collection import SpectrumCollection
+from ..spectra.spectral_axis import SpectralAxis
+
 from ..analysis import (line_flux, equivalent_width, snr, centroid,
-                        gaussian_sigma_width, gaussian_fwhm, fwhm,
+                        gaussian_sigma_width, gaussian_fwhm, fwhm, moment,
                         snr_derived, fwzi, is_continuum_below_threshold)
 from ..fitting import find_lines_threshold
 from ..manipulation import snr_threshold, FluxConservingResampler
@@ -116,7 +119,7 @@ def test_equivalent_width():
     # be negative
     expected = -(np.sqrt(2*np.pi) * u.GHz)
 
-    assert quantity_allclose(result, expected, atol=0.01*u.GHz)
+    assert quantity_allclose(result, expected, atol=0.0025*u.GHz)
 
 
 def test_equivalent_width_masked ():
@@ -144,12 +147,12 @@ def test_equivalent_width_masked ():
 
     assert result.unit.is_equivalent(spectrum.wcs.unit)
 
-    assert quantity_allclose(result.value, -719.90618, atol=0.001)
+    assert quantity_allclose(result.value, -719.90618, atol=0.0005)
 
     # With flux conserving resampler
     result = equivalent_width(spectrum_masked,
                               mask_interpolation=FluxConservingResampler)
-    assert quantity_allclose(result.value, -719.987425, atol=0.001)
+    assert quantity_allclose(result.value, -719.89962, atol=0.0005)
 
 
 def test_equivalent_width_regions():
@@ -157,17 +160,16 @@ def test_equivalent_width_regions():
     np.random.seed(42)
 
     frequencies = np.linspace(100, 1, 10000) * u.GHz
-    g = models.Gaussian1D(amplitude=1*u.Jy, mean=10*u.GHz, stddev=1*u.GHz)
+    g = models.Gaussian1D(amplitude=1*u.Jy, mean=20*u.GHz, stddev=1*u.GHz)
     noise = np.random.normal(0., 0.001, frequencies.shape) * u.Jy
     flux = g(frequencies) + noise + 1*u.Jy
 
     spec = Spectrum1D(spectral_axis=frequencies, flux=flux)
-    cont_norm_spec = spec / np.median(spec.flux)
-    result = equivalent_width(cont_norm_spec, regions=SpectralRegion(97*u.GHz, 3*u.GHz))
+    result = equivalent_width(spec, regions=SpectralRegion(60*u.GHz, 10*u.GHz))
 
     expected = -(np.sqrt(2*np.pi) * u.GHz)
 
-    assert quantity_allclose(result, expected, atol=0.02*u.GHz)
+    assert quantity_allclose(result, expected, atol=0.005*u.GHz)
 
 
 @pytest.mark.parametrize('continuum', [1*u.Jy, 2*u.Jy, 5*u.Jy])
@@ -190,7 +192,7 @@ def test_equivalent_width_continuum(continuum):
     # be negative
     expected = -(np.sqrt(2*np.pi) * u.GHz) / continuum.value
 
-    assert quantity_allclose(result, expected, atol=0.01*u.GHz)
+    assert quantity_allclose(result, expected, atol=0.005*u.GHz)
 
 
 def test_equivalent_width_absorption():
@@ -212,7 +214,41 @@ def test_equivalent_width_absorption():
 
     expected = amplitude*np.sqrt(2*np.pi) * u.GHz
 
-    assert quantity_allclose(result, expected, atol=0.01*u.GHz)
+    assert quantity_allclose(result, expected, atol=0.005*u.GHz)
+
+
+@pytest.mark.parametrize('bin_specification', ["centers", "edges"])
+def test_equivalent_width_bin_edges(bin_specification):
+    """
+    Test spectrum with bin specifications as centers or edges, and at modest sampling.
+    """
+
+    np.random.seed(42)
+
+    wavelengths = np.linspace(500, 2500, 401) * u.AA
+    spectral_axis = SpectralAxis(wavelengths, bin_specification=bin_specification)
+
+    flunit = u.Unit('W m-2 AA-1')
+    amplitude = 0.5
+    stddev = 100*u.AA
+    expected = amplitude*np.sqrt(2*np.pi) * stddev
+
+    g = models.Gaussian1D(amplitude=amplitude*flunit, mean=1000*u.AA, stddev=stddev)
+    continuum = 1 * flunit
+    noise = np.random.normal(0., 0.01, spectral_axis.shape) * flunit
+    flux = continuum - g(spectral_axis) + noise
+
+    spectrum = Spectrum1D(spectral_axis=spectral_axis, flux=flux)
+
+    result = equivalent_width(spectrum)
+
+    assert result.unit.is_equivalent(spectrum.wcs.unit, equivalencies=u.spectral())
+    assert quantity_allclose(result, expected, atol=0.5*u.AA)
+
+    # With SpectralRegion; need higher tolerance as this is cutting more of the wings.
+    result = equivalent_width(spectrum, regions=SpectralRegion(750*u.AA, 1500*u.AA))
+
+    assert quantity_allclose(result, expected, atol=1*u.AA)
 
 
 def test_snr(simulated_spectra):
@@ -921,3 +957,125 @@ def test_is_continuum_below_threshold():
     with pytest.warns(AstropyUserWarning) as e_info:
         find_lines_threshold(spectrum, noise_factor=1)
         assert len(e_info)==1 and 'if you want to suppress this warning' in e_info[0].message.args[0].lower()
+
+
+def test_moment():
+
+    np.random.seed(42)
+
+    frequencies = np.linspace(100, 1, 10000) * u.GHz
+    g = models.Gaussian1D(amplitude=1*u.Jy, mean=10*u.GHz, stddev=1*u.GHz)
+    noise = np.random.normal(0., 0.01, frequencies.shape) * u.Jy
+    flux = g(frequencies) + noise
+
+    spectrum = Spectrum1D(spectral_axis=frequencies, flux=flux)
+
+    moment_0 = moment(spectrum, order=0)
+    assert moment_0.unit.is_equivalent(u.Jy )
+    assert quantity_allclose(moment_0, 252.96*u.Jy, atol=0.01*u.Jy)
+
+    moment_1 = moment(spectrum, order=1)
+    assert moment_1.unit.is_equivalent(u.GHz)
+    assert quantity_allclose(moment_1, 10.08*u.GHz, atol=0.01*u.GHz)
+
+    moment_2 = moment(spectrum, order=2)
+    assert moment_2.unit.is_equivalent(u.GHz**2)
+    assert quantity_allclose(moment_2, 13.40*u.GHz**2, atol=0.01*u.GHz**2)
+
+    moment_3 = moment(spectrum, order=3)
+    assert moment_3.unit.is_equivalent(u.GHz**3)
+    assert quantity_allclose(moment_3, 1233.78*u.GHz**3, atol=0.01*u.GHz**3)
+
+
+def test_moment_cube():
+
+    np.random.seed(42)
+
+    frequencies = np.linspace(100, 1, 10000) * u.GHz
+    g = models.Gaussian1D(amplitude=100*u.Jy, mean=50*u.GHz, stddev=1000*u.GHz)
+    noise = np.random.normal(0., 0.01, frequencies.shape) * u.Jy
+    flux = g(frequencies) + noise
+
+    # use identical arrays in each spaxel. The purpose here is not to
+    # check accuracy (already tested elsewhere), but dimensionality.
+
+    flux_multid = np.broadcast_to(flux, [10,10,flux.shape[0]]) * u.Jy
+
+    spectrum = Spectrum1D(spectral_axis=frequencies, flux=flux_multid)
+
+    moment_1 = moment(spectrum, order=1)
+
+    assert moment_1.shape == (10,10)
+    assert moment_1.unit.is_equivalent(u.GHz)
+    assert quantity_allclose(moment_1, 50.50*u.GHz, atol=0.01*u.GHz)
+
+    # select the last axis of the cube. Should be identical with
+    # the default call above.
+    moment_1_last = moment(spectrum, order=1, axis=2)
+
+    assert moment_1_last.shape == moment_1.shape
+    assert moment_1_last.unit.is_equivalent(moment_1.unit)
+    assert quantity_allclose(moment_1, moment_1_last, rtol=1.E-5)
+
+    # spatial 1st order - returns the dispersion
+    moment_1 = moment(spectrum, order=1, axis=1)
+
+    assert moment_1.shape == (10,10000)
+    assert moment_1.unit.is_equivalent(u.GHz)
+    assert quantity_allclose(moment_1, frequencies, rtol=1.E-5)
+
+    # higher order
+    moment_2 = moment(spectrum, order=2)
+
+    assert moment_2.shape == (10,10)
+    assert moment_2.unit.is_equivalent(u.GHz**2)
+    assert quantity_allclose(moment_2, 816.648*u.GHz**2, atol=0.01*u.GHz**2)
+
+    # spatial higher order (what's the meaning of this?)
+    moment_2 = moment(spectrum, order=2, axis=1)
+
+    assert moment_2.shape == (10,10000)
+    assert moment_2.unit.is_equivalent(u.GHz**2)
+    # check assorted values.
+    assert quantity_allclose(moment_2[0][0], 2.019e-28*u.GHz**2, rtol=0.01)
+    assert quantity_allclose(moment_2[1][0], 2.019e-28*u.GHz**2, rtol=0.01)
+    assert quantity_allclose(moment_2[0][3], 8.078e-28*u.GHz**2, rtol=0.01)
+
+
+def test_moment_collection():
+
+    np.random.seed(42)
+
+    frequencies = np.linspace(100, 1, 10000) * u.GHz
+    g = models.Gaussian1D(amplitude=1*u.Jy, mean=10*u.GHz, stddev=1*u.GHz)
+    noise = np.random.normal(0., 0.01, frequencies.shape) * u.Jy
+    flux = g(frequencies) + noise
+    s1 = Spectrum1D(spectral_axis=frequencies, flux=flux)
+
+    frequencies = np.linspace(200, 2, 10000) * u.GHz
+    g = models.Gaussian1D(amplitude=2*u.Jy, mean=20*u.GHz, stddev=2*u.GHz)
+    noise = np.random.normal(0., 0.02, frequencies.shape) * u.Jy
+    flux = g(frequencies) + noise
+    s2 = Spectrum1D(spectral_axis=frequencies, flux=flux)
+
+    collection = SpectrumCollection.from_spectra([s1, s2])
+
+    # Compare moments derived from collection, with moments
+    # derived from individual members.
+    moment_0 = moment(collection, order=0)
+    moment_0_s1 = moment(s1, order=0)
+    moment_0_s2 = moment(s2, order=0)
+    assert quantity_allclose(moment_0[0], moment_0_s1, rtol=1.E-5)
+    assert quantity_allclose(moment_0[1], moment_0_s2, rtol=1.E-5)
+
+    moment_1 = moment(collection, order=1)
+    moment_1_s1 = moment(s1, order=1)
+    moment_1_s2 = moment(s2, order=1)
+    assert quantity_allclose(moment_1[0], moment_1_s1, rtol=1.E-5)
+    assert quantity_allclose(moment_1[1], moment_1_s2, rtol=1.E-5)
+
+    moment_2 = moment(collection, order=2)
+    moment_2_s1 = moment(s1, order=2)
+    moment_2_s2 = moment(s2, order=2)
+    assert quantity_allclose(moment_2[0], moment_2_s1, rtol=1.E-5)
+    assert quantity_allclose(moment_2[1], moment_2_s2, rtol=1.E-5)
